structure Step :>
sig
    datatype result =
        Done
      | Stepped of Syntax.exp

    (* Given a closed, well-typed expression e (i.e. empty |- e : tau for some tau), 
       returns an Stepped e' for an e' such that e |-> e' if such an e' exists, or 
       returns Done if e is done *)
    val progress : Syntax.exp -> result
        
end = 
struct

    open Syntax

    datatype result =
        Done
      | Stepped of Syntax.exp

    exception DoesntMatch
        
    (* given a pattern p and a done expression e, 
       return theta if p matches e with theta for some theta,
       or raise exception DoesntMatch otherwise *)
    fun match (p : pat, e : exp) : ssubst = raise Fail "unimplemented"

    fun progress e = 
        case e of
            Lam (t,e) => Done
          | App (e1,e2) =>
                     Stepped (case progress e1 of
                                  Done => (case progress e2 of
                                               Done => (case e1 of Lam (t,(x,e)) => subst (e, [(e2,x)]))
                                             | Stepped e2' => App (e1 , e2'))
                                | Stepped e1' => App (e1', e2))
          | Let (e1,(x,e2)) => Stepped (case progress e1 of
                                            Done => (subst (e2, [(e1,x)]))
                                          | Stepped e1' => (Let (e1', (x,e2))))
          | Triv => Done
          | Pair(e1,e2) => (case progress e1 of
                                Done => (case progress e2 of
                                             Done => Done
                                          | Stepped e2' => Stepped (Pair (e1 , e2')))
                              | Stepped e1' => Stepped (Pair (e1', e2)))
          | InLeft (t,e) =>  (case progress e of
                                Done => Done
                              | Stepped e' => Stepped (InLeft (t,e')))
          | InRight (t,e) =>  (case progress e of
                                   Done => Done
                                 | Stepped e' => Stepped (InRight (t,e')))
          | Fold ((a,t),e) =>  raise Fail "unimplemented"
          | Case (e, branches) => raise Fail "unimplemented"
          | Fix(t,(x,e)) => raise Fail "unimplemented"
                                   
end
